<?php
/**
 * Plugin Name: TipTopPay Kassir for WooCommerce
 * Plugin URI: https://tiptoppay.kz/
 * Description: Extends WooCommerce with TipTopPay Kassir.
 * Version: 1.0.0
 * License: GPL v2 or later
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 */
if (!defined('ABSPATH')) exit;

// Register New Order Statuses
function ttkgwwc_register_post_statuses()
{
	register_post_status('wc-pay_delivered', array(
		'label'                     => _x('Доставлен', 'WooCommerce Order status', 'tiptoppay-kassir-for-woocommerce'),
		'public'                    => true,
		'exclude_from_search'       => false,
		'show_in_admin_all_list'    => true,
		'show_in_admin_status_list' => true,
    // translators: order statuses
		'label_count'               => _n_noop('Approved (%s)', 'Approved (%s)', 'tiptoppay-kassir-for-woocommerce')
	));
}

add_filter('init', 'ttkgwwc_register_post_statuses');

// Add New Order Statuses to WooCommerce
function ttkgwwc_add_order_statuses($order_statuses)
{
	$order_statuses['wc-pay_delivered'] = _x('Доставлен', 'WooCommerce Order status', 'tiptoppay-kassir-for-woocommerce');
	return $order_statuses;
}
add_filter('wc_order_statuses', 'ttkgwwc_add_order_statuses');
add_action('plugins_loaded', 'ttkgwwc_TipTopPayKassir', 0);

function ttkgwwc_TipTopPayKassir()
{
	if (!class_exists('WC_Payment_Gateway')) {
		echo 'TipTopPayKassir for WooCommerce plugin is disabled. Check to see if this plugin is active.';
		return;
	}

	add_filter('woocommerce_payment_gateways', 'ttkgwwc_add_ttkgwwc');
	function ttkgwwc_add_ttkgwwc($methods)
	{
		$methods[] = 'TTKGWWC_TipTopPayKassir';
		return $methods;
	}

	class TTKGWWC_TipTopPayKassir extends WC_Payment_Gateway
	{
		public function __construct()
		{
			$this->id                   = 'ttkgwwc_kassir';
			$this->has_fields           = true;
			$this->method_title         = __('TipTopPayKassir', 'tiptoppay-kassir-for-woocommerce');
			$this->method_description   = 'TipTopPayKassir, онлайн-кассы для интернет-магазинов по 54-ФЗ';
			$this->supports             = array('products', 'pre-orders');
			$this->enabled              = $this->get_option('enabled');
			$this->inn                  = $this->get_option('inn');
			$this->status_chancel       = $this->get_option('status_chancel');
			$this->status_pay           = $this->get_option('status_pay');
			$this->init_form_fields();
			$this->init_settings();

			$this->title                = $this->get_option('title');
			$this->description          = $this->get_option('description');
			$this->public_id            = $this->get_option('public_id');
			$this->api_pass             = $this->get_option('api_pass');

			$this->kassa_taxtype        = $this->get_option('kassa_taxtype');
			$this->delivery_taxtype     = $this->get_option('delivery_taxtype');
			$this->kassa_taxsystem      = $this->get_option('kassa_taxsystem');
			$this->calculationPlace     = $this->get_option('calculationPlace');
			$this->kassa_skubarcode     = $this->get_option('kassa_skubarcode');
			$this->kassa_includeshipping= $this->get_option('kassa_includeshipping');
			$this->payment_methods      = $this->get_option('payment_methods');
			$this->kassa_method         = $this->get_option('kassa_method');
			$this->kassa_object         = $this->get_option('kassa_object');
			$this->status_delivered     = $this->get_option('status_delivered');
			$this->shipping_discount    = $this->get_option('shipping_discount') == 'yes';

			add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
			add_action('woocommerce_order_status_changed', array($this, 'ttkgwwc_update_order_status'), 10, 3);
			add_action('woocommerce_api_' . strtolower(get_class($this)), array($this, 'ttkgwwc_callback'));
		}

		// Admin options
		public function admin_options()
		{
            echo '<h3>TipTopPayKassir</h3>';
			echo '<table class="form-table">';
			$this->generate_settings_html();
			echo '</table>';
		}

		// Initialize fields
		public function init_form_fields()
		{
			$array_status = wc_get_order_statuses();

			$ttkgwwc_list_gateways = array(
				'WC_Gateway_BACS',
				'WC_Gateway_Cheque',
				'WC_Gateway_COD',
			);

			$ttkgwwc_list_gateways = apply_filters('woocommerce_payment_gateways', $ttkgwwc_list_gateways);
			$gateways = [];

			foreach ($ttkgwwc_list_gateways as $gateway) {
				if (is_string($gateway) && class_exists($gateway) && $gateway != 'TTKGWWC_TipTopPayKassir') {
					$gateway = new $gateway();
					$gateways[$gateway->id] = $gateway->title;
				}
			}


			$this->form_fields = array(
				'status_pay' => array(
					'title'         => __('Статус для печати чека прихода', 'tiptoppay-kassir-for-woocommerce'),
					'type'          => 'select',
					'class'         => 'wc-enhanced-select',
					'description'   => '',
					'default'       => 'wc-completed',
					'desc_tip'      => true,
					'options'       => $array_status,
				),
				'status_chancel' => array(
					'title'         => __('Статус для печати чека возврата', 'tiptoppay-kassir-for-woocommerce'),
					'type'          => 'select',
					'class'         => 'wc-enhanced-select',
					'description'   => '',
					'default'       => 'wc-cancelled',
					'desc_tip'      => true,
					'options'       => $array_status,
				),
				'title' => array(
					'title'         => __('Title', 'tiptoppay-kassir-for-woocommerce'),
					'type'          => 'text',
					'default'       => __('Онлайн касса', 'tiptoppay-kassir-for-woocommerce'),
					'desc_tip'      => true,
				),
				'description'       => array(
					'title'         => __('Description', 'tiptoppay-kassir-for-woocommerce'),
					'type'          => 'textarea',
					'default'       => __('TipTopPayKassir – онлайн-касса для соответсвия ФЗ-54', 'tiptoppay-kassir-for-woocommerce'),
					'desc_tip'      => true,
				),
				'public_id' => array(
					'title'         => __('Public ID', 'tiptoppay-kassir-for-woocommerce'),
					'type'          => 'text',
					'description'   => 'Возьмите из личного кабинета TipTopPay',
					'default'       => '',
					'desc_tip'      => false,
				),
				'api_pass' => array(
					'title'         => __('Пароль для API', 'tiptoppay-kassir-for-woocommerce'),
					'type'          => 'text',
					'description'   => 'Возьмите из личного кабинета TipTopPay',
					'default'       => '',
					'desc_tip'      => false,
				),
				'inn' => array(
					'title'         => __('ИИН/БИН', 'tiptoppay-kassir-for-woocommerce'),
					'type'          => 'text',
					'description'   => '',
					'default'       => '',
					'desc_tip'      => true,
				),
				'calculationPlace' => array(
					'title'         => __('Место осуществления расчёта', 'tiptoppay-kassir-for-woocommerce'),
					'type'          => 'text',
					'description'   => '',
					'default'       => '',
					'desc_tip'      => true,
				),
				'kassa_taxtype' => array(
					'title'         => __('Ставка НДС', 'tiptoppay-kassir-for-woocommerce'),
					'type'          => 'select',
					'class'         => 'wc-enhanced-select',
					'description'   => __('Выберите ставку НДС, которая применима к товарам в магазине.', 'tiptoppay-kassir-for-woocommerce'),
					'default'       => '10',
					'desc_tip'      => true,
					'options'       => array(
						''      => __('НДС не облагается', 'tiptoppay-kassir-for-woocommerce'),
						'12'    => __('НДС 12%', 'tiptoppay-kassir-for-woocommerce'),
						'10'    => __('НДС 10%', 'tiptoppay-kassir-for-woocommerce'),
						'0'     => __('НДС 0%', 'tiptoppay-kassir-for-woocommerce')
					),
				),
				'delivery_taxtype' => array(
					'title'         => __('Ставка НДС для доставки', 'tiptoppay-kassir-for-woocommerce'),
					'type'          => 'select',
					'class'         => 'wc-enhanced-select',
					'description'   => __('Выберите ставку НДС, которая применима к доставке в магазине.', 'tiptoppay-kassir-for-woocommerce'),
					'default'       => '10',
					'desc_tip'      => true,
					'options'       => array(
						''      => __('НДС не облагается', 'tiptoppay-kassir-for-woocommerce'),
						'12'    => __('НДС 12%', 'tiptoppay-kassir-for-woocommerce'),
						'10'    => __('НДС 10%', 'tiptoppay-kassir-for-woocommerce'),
						'0'     => __('НДС 0%', 'tiptoppay-kassir-for-woocommerce')
					),
				),
				'kassa_taxsystem' => array(
					'title'       => __('Cистема налогообложения организации', 'tiptoppay-kassir-for-woocommerce'),
					'type'        => 'select',
					'class'       => 'wc-enhanced-select',
					'description' => __('Указанная система налогообложения должна совпадать с одним из вариантов, зарегистрированных в ККТ.', 'tiptoppay-kassir-for-woocommerce'),
					'default'     => '1',
					'desc_tip'    => true,
					'options'     => array(
            '0' => __('Общеустановленный режим налогообложения', 'tiptoppay-kassir-for-woocommerce'),
            '1' => __('СНР на основе упрощенной декларации', 'tiptoppay-kassir-for-woocommerce'),
            '4' => __('СНР на основе патента', 'tiptoppay-kassir-for-woocommerce'),
					),
				),
				'kassa_method' => array(
					'title'       => __('Способ расчета', 'tiptoppay-kassir-for-woocommerce'),
					'type'        => 'select',
					'class'       => 'wc-enhanced-select',
					'description' => __('Выберите способ расчета', 'tiptoppay-kassir-for-woocommerce'),
					'default'     => '1',
					'desc_tip'    => true,
					'options'     => array(
						'0' => __('Способ расчета не передается', 'tiptoppay-kassir-for-woocommerce'),
						'1' => __('Предоплата 100%', 'tiptoppay-kassir-for-woocommerce'),
						'2' => __('Предоплата', 'tiptoppay-kassir-for-woocommerce'),
						'3' => __('Аванс', 'tiptoppay-kassir-for-woocommerce'),
						'4' => __('Полный расчёт', 'tiptoppay-kassir-for-woocommerce'),
						'5' => __('Частичный расчёт и кредит', 'tiptoppay-kassir-for-woocommerce'),
						'6' => __('Передача в кредит', 'tiptoppay-kassir-for-woocommerce'),
						'7' => __('Оплата кредита', 'tiptoppay-kassir-for-woocommerce'),
					),
				),
				'kassa_object' => array(
					'title'       => __('Предмет расчета', 'tiptoppay-kassir-for-woocommerce'),
					'type'        => 'select',
					'class'       => 'wc-enhanced-select',
					'description' => __('Выберите предмет расчета', 'tiptoppay-kassir-for-woocommerce'),
					'default'     => '1',
					'desc_tip'    => true,
					'options'     => array(
						'0' => __('Предмет расчета не передается', 'tiptoppay-kassir-for-woocommerce'),
						'1' => __('Товар', 'tiptoppay-kassir-for-woocommerce'),
						'2' => __('Подакцизный товар', 'tiptoppay-kassir-for-woocommerce'),
						'3' => __('Работа', 'tiptoppay-kassir-for-woocommerce'),
						'4' => __('Услуга', 'tiptoppay-kassir-for-woocommerce'),
						'5' => __('Ставка азартной игры', 'tiptoppay-kassir-for-woocommerce'),
						'6' => __('Выигрыш азартной игры', 'tiptoppay-kassir-for-woocommerce'),
						'7' => __('Лотерейный билет', 'tiptoppay-kassir-for-woocommerce'),
						'8' => __('Выигрыш лотереи', 'tiptoppay-kassir-for-woocommerce'),
						'9' => __('Предоставление РИД', 'tiptoppay-kassir-for-woocommerce'),
						'10' => __('Платеж', 'tiptoppay-kassir-for-woocommerce'),
						'11' => __('Агентское вознаграждение', 'tiptoppay-kassir-for-woocommerce'),
						'12' => __('Составной предмет расчета', 'tiptoppay-kassir-for-woocommerce'),
						'13' => __('Иной предмет расчета', 'tiptoppay-kassir-for-woocommerce'),
					),
				),
				'status_delivered' => array(
					'title'       => __('Статус которым пробивать 2ой чек при отгрузке товара или выполнении услуги', 'tiptoppay-kassir-for-woocommerce'),
					'type'        => 'select',
					'class'       => 'wc-enhanced-select',
					'description' => __('Согласно ФЗ-54 владельцы онлайн-касс должны формировать чеки для зачета и предоплаты. Отправка второго чека возможна только при следующих способах расчета: Предоплата, Предоплата 100%, Аванс', 'tiptoppay-kassir-for-woocommerce'),
					'default'     => 'wc-pay_delivered',
					'desc_tip'    => true,
					'options'     => $array_status,
				),
				'payment_methods' => array(
					'title'       => __('Для каких платежных систем включать', 'tiptoppay-kassir-for-woocommerce'),
					'type'        => 'multiselect',
					'class'       => 'wc-enhanced-select',
					'description' => '',
					'default'     => '',
					'multiple'    => true,
					'desc_tip'    => true,
					'options'     => array(),
				),
				'shipping_discount' => array(
					'title'       => __('Применять скидку к доставке', 'tiptoppay-kassir-for-woocommerce'),
					'type'        => 'checkbox',
					'description' => __('Если включено, скидка к доставке применяется', 'tiptoppay-kassir-for-woocommerce'),
					'desc_tip'    => true,
					'label'       => ' ',
				)
			);

			foreach ($gateways as $paym => $k)
      {
        $this->form_fields['payment_methods']['options'][$paym] = $k;
      }
		}

		public function ttkgwwc_get_order($request)
		{
			global $woocommerce;
			$order = new WC_Order($request['InvoiceId']);
			return $order;
		}

		public function ttkgwwc_SendReceipt($order, $type, $old_status, $new_status)
		{
			$cart = $order->get_items();
			$total_amount = 0;
			$total_amount_full = 0;
			$total_fees = 0;

			foreach ($cart as $item_id => $item_data):
				if ($item_data->is_type('shipping') || $item_data->is_type('fee')) continue;

				$product = $item_data->get_product();
				if ("wc-" . $new_status == $this->status_delivered) $method = 4;
                else $method = (int)$this->kassa_method;

				//вычисление скидки
				$fees = $order->get_items('fee');
				foreach ($fees as $fee) { $total_fees = floatval($fee->get_total());}


				$items[] = array(
					'label'    => $product->get_name(),
					'price'    => number_format($product->get_price(), 2, ".", ''),
					'quantity' => $item_data->get_quantity(),
					'amount'   => number_format(floatval($item_data->get_total()), 2, ".", ''),
					'vat'      => $this->kassa_taxtype,
					'method'   => $method,
					'object'   => (int)$this->kassa_object,
				);

				$total_amount = $total_amount + number_format(floatval($item_data->get_total()), 2, ".", '');
				$total_amount_full = $total_amount_full + number_format(floatval($item_data->get_total()), 2, ".", '');

			endforeach;

			if ($order->get_total_shipping()):
				$items[] = array(
					'label'    => "Доставка",
					'price'    => $order->get_total_shipping(),
					'quantity' => 1,
					'amount'   => $order->get_total_shipping(),
					'vat'      => $this->delivery_taxtype,
					'method'   => $method,
					'object'   => 4,
				);

				if ($this->shipping_discount) $total_amount = $total_amount + number_format(floatval($order->get_total_shipping()), 2, ".", '');
				$total_amount_full = $total_amount_full + number_format(floatval($order->get_total_shipping()), 2, ".", '');

			endif;


			$data['PaymentData']['customerReceipt']['Items']            = $items;
			$data['PaymentData']['customerReceipt']['taxationSystem']   = $this->kassa_taxsystem;
			$data['PaymentData']['customerReceipt']['calculationPlace'] = $this->calculationPlace;
			$data['PaymentData']['customerReceipt']['email']            = $order->get_billing_email();
			$data['PaymentData']['customerReceipt']['phone']            = $order->get_billing_phone();
			//вычисление итогового amount
			$data['PaymentData']['customerReceipt']['amounts']['electronic'] = $total_amount_full + $total_fees;

			//вычитаем скидку
			if ($total_fees > 0) $percent = $total_fees / floatval($total_amount);

			$total = 0;
			foreach ($data['PaymentData']['customerReceipt']['Items'] as &$item) {
				if (!empty($percent) && ($item['label'] != 'Доставка' || $this->shipping_discount)) {
					$fee_item = floatval($item['amount']) * $percent;

					$amount = $item['amount'] + $fee_item;
					$item['amount'] = number_format(floatval($amount), 2, ".", '');
				}
				$total += $item['amount'];
			}

			if ($total != $data['PaymentData']['customerReceipt']['amounts']['electronic']) {
				$diff = $total - $data['PaymentData']['customerReceipt']['amounts']['electronic'];
				$data['PaymentData']['customerReceipt']['Items'][0]['amount'] -= number_format($diff, 2, ".", '');
			}

			//вычитаем скидку из второго чека
			if ("wc-" . $new_status == $this->status_delivered) {
				$data['PaymentData']['customerReceipt']['amounts']['electronic'] = 0;
				$data['PaymentData']['customerReceipt']['amounts']['advancePayment'] = $total_amount + $total_fees;
			}

			$aData = array(
				'Inn'             => $this->inn,
				'InvoiceId'       => $order->get_id(), //номер заказа, необязательный
				'AccountId'       => $order->get_user_id(),
				'Type'            => $type,
				'CustomerReceipt' => $data['PaymentData']['customerReceipt']
			);

			$API_URL = 'https://api.tiptoppay.kz/kkt/receipt';
			self::ttkgwwc_send_request($API_URL, $aData);
		}

		public function ttkgwwc_send_request($API_URL, $request)
		{
			$request2 = self::ttkgwwc_curl_json_encode($request);
			$str = gmdate("d-m-Y H:i:s") . $request['Type'] . $request['InvoiceId'] . $request['AccountId'] . $request['CustomerReceipt']['email'];
			$reque = md5($str);
			$auth = base64_encode($this->public_id . ":" . $this->api_pass);
			$response = wp_remote_post($API_URL, array(
				'timeout'     => 30,
				'redirection' => 5,
				'httpversion' => '1.0',
				'blocking'    => true,
				'headers'     => array('Authorization' => 'Basic ' . $auth, 'Content-Type' => 'application/json', 'X-Request-ID' => $reque),
				'body'        => $request2,
				'cookies'     => array()
			));

			$body = json_decode($response['body']);
		}

		public function ttkgwwc_update_order_status($order_id, $old_status, $new_status) //OK
		{
			$request['InvoiceId'] = $order_id;
			$order = self::ttkgwwc_get_order($request);
			if (!empty($this->payment_methods) && in_array($order->get_payment_method(), $this->payment_methods)):
				if ($this->status_pay == "wc-" . $new_status || ("wc-" . $new_status == $this->status_delivered && in_array((int)$this->kassa_method, array(1, 2, 3)))):
					self::ttkgwwc_SendReceipt($order, 'Income', $old_status, $new_status);
                elseif ($this->status_chancel == "wc-" . $new_status):
					self::ttkgwwc_SendReceipt($order, 'IncomeReturn', $old_status, $new_status);
				endif;
			endif;
		}


		function ttkgwwc_curl_json_encode($a = false)
		{
			if (is_null($a) || is_resource($a)) return 'null';
			if ($a === false) return 'false';
			if ($a === true) return 'true';

			if (is_scalar($a)) {
				if (is_float($a)) $a = str_replace(',', '.', strval($a));

				static $jsonReplaces = array(
					array("\\", "/", "\n", "\t", "\r", "\b", "\f", '"'),
					array('\\\\', '\\/', '\\n', '\\t', '\\r', '\\b', '\\f', '\"')
				);

				return '"' . str_replace($jsonReplaces[0], $jsonReplaces[1], $a) . '"';
			}

			$isList = true;

			for ($i = 0, reset($a); $i < count($a); $i++, next($a)) {
				if (key($a) !== $i) {
					$isList = false;
					break;
				}
			}

			$result = array();

			if ($isList) {
				foreach ($a as $v) { $result[] = self::ttkgwwc_curl_json_encode($v);}

				return '[ ' . join(', ', $result) . ' ]';
			} else {
				foreach ($a as $k => $v) {$result[] = self::ttkgwwc_curl_json_encode($k) . ': ' . self::ttkgwwc_curl_json_encode($v);}

				return '{ ' . join(', ', $result) . ' }';
			}
		}

		public function ttkgwwc_callback()
		{
      if (empty($_GET['action']) or empty($_POST['InvoiceId']) or empty($_POST['Type']) or empty($_POST['Url'])) {
        exit;
      }

			if (sanitize_text_field(wp_unslash($_GET['action'])) == 'receipt') {

				$request['InvoiceId'] = sanitize_text_field(wp_unslash($_POST['InvoiceId']));
        $type = sanitize_text_field(wp_unslash($_POST['Type'])) == 'IncomeReturn' ? 'возврата прихода' :'прихода';
				$url = sanitize_text_field(wp_unslash($_POST['Url']));
				$note = 'Ссылка на чек ' . $type . ': ' . $url;
				$order = self::ttkgwwc_get_order($request);
				$order->add_order_note($note, 1);
				$order->save();
				echo wp_json_encode(['CODE' => 0]);
				exit;
			}
		}
	}
}